#ifndef GREADER_GCLIENT_H
#define GREADER_GCLIENT_H


#include "delegate.h"
#include "message.h"
#include <semaphore.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    EMESS_AppGetReaderInfo = 0,
    EMESS_AppGetBaseVersion = 1,
    EMESS_AppSetSerialParam = 2,
    EMESS_AppGetSerialParam = 3,
    EMESS_AppSetEthernetIp = 4,
    EMESS_AppGetEthernetIp = 5,
    EMESS_AppGetEthernetMac = 6,
    EMESS_AppSetTcpMode = 7,
    EMESS_AppGetTcpMode = 8,
    EMESS_AppSetGpo = 9,
    EMESS_AppGetGpiState = 10,
    EMESS_AppSetGpiTrigger = 11,
    EMESS_AppGetGpiTrigger = 12,
    EMESS_AppSetWiegand = 13,
    EMESS_AppGetWiegand = 14,
    EMESS_AppReboot = 15,
    EMESS_AppSetReaderTime = 16,
    EMESS_AppGetReaderTime = 17,
    EMESS_AppHeartbeat = 18,
    EMESS_AppSetReaderMac = 19,
    EMESS_AppRestoreDefault = 20,
    EMESS_AppSetRs485 = 21,
    EMESS_AppGetRs485 = 22,
    EMESS_AppSetBreakpointResume = 23,
    EMESS_AppGetBreakpointResume = 24,
    EMESS_AppGetCacheTagData = 25,
    EMESS_AppClearCacheTagData = 26,
    EMESS_AppTagDataReply = 27,
    EMESS_AppSetBuzzerSwitch = 28,
    EMESS_AppSetBuzzerCtrl = 29,
    EMESS_AppGetWhiteList = 30,
    EMESS_AppImportWhiteList = 31,
    EMESS_AppDeleteWhiteList = 32,
    EMESS_AppSetWhiteListAction = 33,
    EMESS_AppGetWhiteListAction = 34,
    EMESS_AppSetWhiteListSwitch = 35,
    EMESS_AppGetWhiteListSwitch = 36,
    EMESS_AppSetUdpParam = 37,
    EMESS_AppGetUdpParam = 38,
    EMESS_AppSetHttpParam = 39,
    EMESS_AppGetHttpParam = 40,
    EMESS_AppUsbKbdSwitch = 41,
    EMESS_AppSetWifiSearch = 42,
    EMESS_AppGetWifiSearch = 43,
    EMESS_AppSetWifiParam = 44,
    EMESS_AppGetWifiStatus = 45,
    EMESS_AppSetWifiIp = 46,
    EMESS_AppGetWifiIp = 47,
    EMESS_AppSetWifiOnOff = 48,
    EMESS_AppGetWifiOnOff = 49,
    EMESS_AppSetEasAlarm = 50,
    EMESS_AppGetEasAlarm = 51,
    EMESS_AppWifiRoaming = 52,
    EMESS_AppWatchDog = 53,

    EMESS_BaseGetCapabilities,
    EMESS_BaseSetPower,
    EMESS_BaseGetPower,
    EMESS_BaseSetFreqRange,
    EMESS_BaseGetFreqRange,
    EMESS_BaseSetFrequency,
    EMESS_BaseGetFrequency,
    EMESS_BaseSetAntennaHub,
    EMESS_BaseGetAntennaHub,
    EMESS_BaseSetTagLog,
    EMESS_BaseGetTagLog,
    EMESS_BaseSetBaseband,
    EMESS_BaseGetBaseband,
    EMESS_BaseSetAutoDormancy,
    EMESS_BaseGetAutoDormancy,
    EMESS_BaseSetResidenceTime,
    EMESS_BaseGetResidenceTime,
    EMESS_BaseSetGbBaseband,
    EMESS_BaseGetGbBaseband,
    EMESS_BaseSetGjbBaseband,
    EMESS_BaseGetGjbBaseband,
    EMESS_BaseInventoryEpc,
    EMESS_BaseWriteEpc,
    EMESS_BaseLockEpc,
    EMESS_BaseDestroyEpc,
    EMESS_BaseMonzaQt,
    EMESS_BaseSuperRW,
    EMESS_BaseInventory6b,
    EMESS_BaseWrite6b,
    EMESS_BaseLock6b,
    EMESS_BaseLockGet6b,
    EMESS_BaseInventoryGb,
    EMESS_BaseWriteGb,
    EMESS_BaseLockGb,
    EMESS_BaseDestroyGb,
    EMESS_BaseInventoryGjb,
    EMESS_BaseWriteGjb,
    EMESS_BaseLockGjb,
    EMESS_BaseDestroyGjb,
    EMESS_BaseSafeCertification,
    EMESS_BaseInventoryHybrid,
    EMESS_BasePassthrough,
    EMESS_BaseStop,

    EMESS_UpgradeApp,
    EMESS_UpgradeBaseband,

    EMESS_TestInitWorkingMode,

    EMESS_Count
} MESSAGE;

typedef enum {
    ETagEpcLog = 0,
    ETagEpcOver = 1,
    ETag6bLog = 2,
    ETag6bOver = 3,
    ETagGbLog = 4,
    ETagGbOver = 5,
    ETagGjbLog = 6,
    ETagGjbOver = 7,
    EGpiStart = 8,
    EGpiOver = 9,
    ETcpDisconnected = 10,
    EGClientConnected = 11,
    EUsbHidRemoved = 12,
} Callback_Type;

typedef enum {
    RS232 = 1,
    RS485 = 2,
    ETH = 3,
    SERVER = 4,
    ACCEPT = 5,
    USB = 6,
    OTHER = 1000
} ConnType;

typedef struct {
    void *rst;
} MessageResult;

typedef void(*delegateOnRemoteConnected)(char *readerName, int handleSocket);

typedef struct {
    long handleSocket;
    delegateOnRemoteConnected onRemoteConnected;
}TcpServer;

typedef struct {
    char name[128];
    char serialNumber[21];
    long handle;
    delegateTagEpcLog call_TagEpcLog;
    delegateTagEpcOver call_TagEpcOver;
    delegateTag6bLog call_Tag6bLog;
    delegateTag6bOver call_Tag6bOver;
    delegateTagGbLog call_TagGbLog;
    delegateTagGbOver call_TagGbOver;
    delegateTagGjbLog call_TagGjbLog;
    delegateTagGjbOver call_TagGjbOver;
    delegateGpiStart call_GpiStart;
    delegateGpiOver call_GpiOver;
    delegateWorkingModeInit call_WorkingModeInit;
    delegateTcpDisconnected call_TcpDisconnected;
    delegateUsbHidRemoved call_UsbHidRemoved;
    sem_t *sem;
    MessageResult *result;
    pthread_mutex_t mutex;
    TcpServer *tcpServer;
    int initParamLen;
    char initParam[1024];
    unsigned char *data;
    int index;
    ConnType type;
    int rs485Address;
    bool isPrint;
    bool isOpened;
    bool threadIsStop;
    bool timerThreadIsStop;
    unsigned long tick;
    pthread_t tid;
} GClient;

GClient *OpenRS232(const char *readerName, int timeout);
GClient *OpenRS485(const char *readerName, int timeout);
GClient *OpenClient(char *readerName, int handSocket);
GClient *OpenTcpClient(const char *readerName, int timeout);
GClient *OpenUSBHID(int timeout);

void RegCallBack(GClient *client, Callback_Type type, void *call);
void SendSynMsgTimeout(GClient *client, MESSAGE type, void *msg, int timeout);
void SendSynMsg(GClient *client, MESSAGE type, void *msg);
void SendSynMsgTimeoutRetry(GClient *client, MESSAGE type, void *msg, int timeout, int retry);
void SendUnSynMsg(GClient *client, MESSAGE type, void *msg);
void SendUnSynMsgTimeoutRetry(GClient *client, MESSAGE type, void *msg, int retry);
void Close(GClient *client);

#ifdef __cplusplus
}
#endif

#endif //GREADER_GCLIENT_H
